/*


Nixiuhr mit HH:MM:SS Anzeige ohne Multiplexing

ATmega328 mit 32kHz Uhrenquarz + interner 8 MHz RC-Oszillator

*/

#include <avr/io.h>
#include <avr/sleep.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>
#include "nixi.h"
#include "functions.h"
#include <util/delay.h>

// globale Variablen
 
// beidseitiger Zugriff durch ISR und Hauptprogramm
 
volatile uint32_t time;             // 24h Zeitstempel, 1s Auflsung
volatile int16_t rtc_cal;           // Kalibrierung des 32K Quarzes
volatile uint8_t flag_1s;           // Flag fr 1s Intervall

uint8_t nixi[9];                    // Schieberegisterdaten

// EEPROM Daten
 
int16_t ee_rtc_cal EEMEM = (F_CAL-12800000)*256/100;        // Kalibrierung fr RTC, Frequenzfehler in 1/1000 Hz
  
int main(void) {

    // init IOs

    PORTC = ~(1<<PC0);      // pull UPs ON for PC1-PC5

    DDRD = (1<<PD6);        // PD6 = NCS_FLASH
    PORTD = 0xFF;           // pull UPs ON for PC0-PC5, PC7, NCS_FLASH HIGH

    DDRB = (1<<PD5) | (1<<PD3) | (1<<PD2) | (1<<PD1);
    PORTB = 0xFF;

    // init SPI

    SPCR = (1<<SPE) | (1<<MSTR);

    clear_all();
    update_nixi();

 // Analogcomparator ausschalten
 
    ACSR = 0x80;
 
// Timer2 konfigurieren

    ASSR   = (1<< AS2);             // Timer2 asynchron takten
    _delay_ms(1000);                // Einschwingzeit des 32kHz Quarzes
    TCCR2A = (1<<WGM21) | (1<<WGM20); // Fast PWM, non inverted
//    TCCR2A = (1<<COM2A1) | (1<<WGM21) | (1<<WGM20); // Fast PWM, non inverted
    TCCR2B = 1;                     // Vorteiler 1 -> 7,8ms berlaufperiode
    OCR2A  = 128;                   // PWM, Tastverhltnis 50%
    while((ASSR & (1<< TCR2BUB)));  // Warte auf das Ende des Zugriffs
    TIFR2  &= ~(1<<TOV2);           // Interrupts lschen
    TIMSK2 |= (1<<TOIE2);           // Timer overflow Interrupt freischalten

// EEPROM Werte auslesen
 
    rtc_cal= eeprom_read_word((uint16_t*)&ee_rtc_cal);

    uint8_t i, j;

    // Einschalttest, alle Ziffern einzeln anzeigen

    for (j=0; j<10; j++) {
        for (i=0; i<6; i++) {
            set_number(i, j);
        }
        update_nixi();
        for (i=0; i<6; i++) {
            clear_digit(i);
        }
        _delay_ms(200);
    }
    set_number(6, 1);      // DOT 1
    set_number(6, 2);      // DOT 2
    update_nixi();
    _delay_ms(200);

    //  Interrupts freigeben
     
    sei();

    flag_1s = 1;

    // endlose Hauptschleife

    while(1) {

        while((ASSR & (1<< OCR2AUB)));  // Warte auf das Ende des Zugriffs
        set_sleep_mode(SLEEP_MODE_PWR_SAVE);
        sleep_mode();      

        if (flag_1s) {  
            flag_1s =0;
            make_time();
            update_nixi();
        }
    }
}

// Timer2 overflow Interrupt
 
ISR(TIMER2_OVF_vect) {
    static uint8_t ticks;               // Hilfsvariable fr Messintervall
    static int16_t time_error;          // RTC Fehlerkompensation
 
    // Zeitkritische Dinge, welche am Anfang der ISR stehen mssen!
 
    OCR2A=128;                          // Dummy-Write zur Sicherung des Timings
                                        // von Timer 2 im asynchronen Modus 

    // RTC Fehler korrigieren
 
    if (time_error>999) {               // RTC zu schnell
        TCNT2 = 2;                      // Zhler einen Schritt zurck setzen (2 Takte Verzgerung!)
        time_error -= 1000;
    } else if (time_error<-999) {       // RTC zu langsam
        TCNT2 = 4;                      // Zhler einen Schritt vor setzen (2 Takte Verzgerung!)
        time_error += 1000;     
    }
 
    // ab hier ist es nicht mehr zeitkritisch
 
    // Echtzeituhr
    ticks++;                            // 1/128tel Sekunde
    if (ticks==128) {                   // Sekundenintervall
        time_error += rtc_cal;          // RTC Fehler akkumulieren
 
        // 24h Timer
        time++;
        if (time==86400) time=0;        // 24h berlauf
 
        ticks=0;
        flag_1s =1;                     // setzte Flag fr 1s Verarbeitung in main Endlosschleife
    }
}
