#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include "nixi.h"
#include <util/delay.h>

// Bitnummern fr Segmente
// verwrfelte Bitzuordnung zur Erleichterung des Hardwarelayouts

const uint8_t segments[7][10] PROGMEM = 
        {{21, 19, 18, 17,  7,  6,  5,  4,  3, 20},  // linke Anzeige
         {11,  9, 23, 22,  2,  1, 15, 14, 13, 10},
         {45, 43, 42, 41, 31, 30, 29, 28, 27, 44},
         {35, 33, 47, 46, 26, 25, 39, 38, 37, 34},
         {69, 67, 66, 65, 55, 54, 53, 52, 51, 68},
         {59, 57, 71, 70, 50, 49, 63, 62, 61, 58},  // rechte Anzeige
         { 0, 12, 36,  0,  0,  0,  0,  0,  0,  0}}; // DOT 1&2

// setzt eine Zahl einer einzelnen Stelle
// bisherige Zahlen bleiben unverndert, somit knnen auch mehrere Segmente gleichzeitig leuchten

void set_number(uint8_t digit, uint8_t number) {
    uint8_t byte, bit, bitnumber;

    bitnumber = pgm_read_byte(&segments[digit][number]);
    byte = bitnumber / 8;
    bit  = bitnumber % 8;
    nixi[byte] |= (1<<bit);
}

// lscht alle Zahlen einer Stelle

void clear_digit(uint8_t digit) {
    uint8_t byte, bit, i, bitnumber;

    for (i=0; i<10; i++) {
        bitnumber = pgm_read_byte(&segments[digit][i]);
        byte = bitnumber / 8;
        bit  = bitnumber % 8;
        nixi[byte] &= ~(1<<bit);
    }
}

// lscht alle Anzeigedaten

void clear_all(void) {
    uint8_t i;

    for (i=0; i<sizeof(nixi); i++) {
        nixi[i] = 0;
    }
}

// Anzeige aktualisieren

void update_nixi(void) {
    int8_t i;

    NCS_SER_LOW
    for (i=8; i>=0; i--) {
        SPDR=nixi[i];
        while( !(SPSR & (1<<SPIF)) );
    }
    NCS_SER_HIGH
}

// globale Zeit von 32 Bit int in hh:mm:ss umrechnen

void make_time(void) {

    uint8_t h,m,s;
    uint32_t time_local;

    clear_all();
        
    // get time, atomic

    cli();
    time_local = time;
    sei();

    // convert to hh:mm:ss 

    s = time_local % 60;
    time_local /= 60;
    m = time_local % 60;
    time_local /= 60;
    h = time_local;

    // convert time to nixi code

    set_number(5, s%10);
    set_number(4, s/10);
    set_number(3, m%10);
    set_number(2, m/10);
    set_number(1, h%10);
    set_number(0, h/10);
    set_number(6, 1);      // DOT 1
    set_number(6, 2);      // DOT 2
}

